from datetime import datetime, timedelta
from typing import Union
from django.http import HttpRequest
from django.utils.timezone import now

def seconds_until_session_end(
    request: HttpRequest,
    session_time: Union[int, timedelta],
    current_time: datetime
) -> float:
    """
    Get seconds until the end of the session.
    :param request: django.http.HttpRequest
    :param session_time: int - for seconds | timedelta
    :param current_time: datetime - use django_auto_logout.utils.now
    :return: float
    """
    if isinstance(session_time, timedelta):
        ttl = session_time
    elif isinstance(session_time, int):
        ttl = timedelta(seconds=session_time)
    else:
        raise TypeError(f"AUTO_LOGOUT['SESSION_TIME'] should be `int` or `timedelta`, "
                        f"not `{type(session_time).__name__}`.")

    return (request.user.last_login - current_time + ttl).total_seconds()

def seconds_until_idle_time_end(
    request: HttpRequest,
    idle_time: Union[int, timedelta],
    current_time: datetime
) -> float:
    """
    Get seconds until the end of downtime.
    :param request: django.http.HttpRequest
    :param idle_time: int - for seconds | timedelta
    :param current_time: datetime - use django_auto_logout.utils.now
    :return: float
    """
    if isinstance(idle_time, timedelta):
        ttl = idle_time
    elif isinstance(idle_time, int):
        ttl = timedelta(seconds=idle_time)
    else:
        raise TypeError(f"AUTO_LOGOUT['IDLE_TIME'] should be `int` or `timedelta`, "
                        f"not `{type(idle_time).__name__}`.")

    last_request_time = request.session.get('django_auto_logout_last_request', current_time.isoformat())
    last_request_time = datetime.fromisoformat(last_request_time)

    return (last_request_time - current_time + ttl).total_seconds()


from django.conf import settings
from django.utils.safestring import mark_safe
from django.http import HttpRequest
from .utils import now, seconds_until_session_end, seconds_until_idle_time_end

LOGOUT_TIMEOUT_SCRIPT_PATTERN = """
<script>
    (function() {
        var w = window,
            s = w.localStorage,
        %s
        w.addEventListener('load', function() {
            s['djalLogoutAt'] = at;
            
            function upd() {
                if (s['djalLogoutAt'] > at) {
                    at = s['djalLogoutAt'];
                    setTimeout(upd, at - Date.now());
                }
                else {
                    delete s['djalLogoutAt'];
                    w.location.reload();
                }
            }
            
            setTimeout(upd, at - Date.now());
        });
    })();
</script>
"""

def _trim(s: str) -> str:
    return ''.join([line.strip() for line in s.split('\n')])

def auto_logout_client(request: HttpRequest) -> dict:
    if request.user.is_anonymous:
        return {}

    options = getattr(settings, 'AUTO_LOGOUT', None)
    if not options:
        return {}

    ctx = {}
    current_time = now()

    if 'SESSION_TIME' in options:
        ctx['seconds_until_session_end'] = seconds_until_session_end(request, options['SESSION_TIME'], current_time)

    if 'IDLE_TIME' in options:
        ctx['seconds_until_idle_end'] = seconds_until_idle_time_end(request, options['IDLE_TIME'], current_time)

    if options.get('REDIRECT_TO_LOGIN_IMMEDIATELY'):
        at = None

        if 'SESSION_TIME' in options and 'IDLE_TIME' in options:
            at = (
                f"at=Date.now()+Math.max(Math.min({ ctx['seconds_until_session_end'] },"
                f"{ ctx['seconds_until_idle_end'] }),0)*1000+999;"
            )
        elif 'SESSION_TIME' in options:
            at = f"at=Date.now()+Math.max({ ctx['seconds_until_session_end'] },0)*1000+999;"
        elif 'IDLE_TIME' in options:
            at = f"at=Date.now()+Math.max({ ctx['seconds_until_idle_end'] },0)*1000+999;"

        if at:
            ctx['redirect_to_login_immediately'] = mark_safe(_trim(LOGOUT_TIMEOUT_SCRIPT_PATTERN % at))

    return ctx


import logging
from typing import Callable
from django.conf import settings
from django.http import HttpRequest, HttpResponse
from django.contrib.auth import get_user_model, logout
from django.contrib.messages import info

from .utils import now, seconds_until_idle_time_end, seconds_until_session_end

UserModel = get_user_model()
logger = logging.getLogger(__name__)

def _auto_logout(request: HttpRequest, options: dict) -> None:
    should_logout = False
    current_time = now()

    if 'SESSION_TIME' in options:
        session_time = seconds_until_session_end(request, options['SESSION_TIME'], current_time)
        should_logout |= session_time < 0
        logger.debug('Check SESSION_TIME: %ss until session ends.', session_time)

    if 'IDLE_TIME' in options:
        idle_time = seconds_until_idle_time_end(request, options['IDLE_TIME'], current_time)
        should_logout |= idle_time < 0
        logger.debug('Check IDLE_TIME: %ss until idle ends.', idle_time)

        if should_logout and 'django_auto_logout_last_request' in request.session:
            del request.session['django_auto_logout_last_request']
        else:
            request.session['django_auto_logout_last_request'] = current_time.isoformat()

    if should_logout:
        logger.debug('Logout user %s', request.user)
        logout(request)

        if 'MESSAGE' in options:
            info(request, options['MESSAGE'])

def auto_logout(get_response: Callable[[HttpRequest], HttpResponse]) -> Callable[[HttpRequest], HttpResponse]:
    def middleware(request: HttpRequest) -> HttpResponse:
        if not request.user.is_anonymous and hasattr(settings, 'AUTO_LOGOUT'):
            _auto_logout(request, settings.AUTO_LOGOUT)

        return get_response(request)
    return middleware


from setuptools import setup

with open('README.md') as arq:
    readme = arq.read()

setup(name='auto-logout-django',
    version='1.0.5',
    author ='José Carlos Tenório',
    long_description=readme,
    long_description_content_type='text/markdown',
    author_email='jcsilva.tenorio@gmail.com',
    keywords='django auto logout',
    description=u'Django auto logout não oficial para Django',
    packages=['auto-logout-django'],
    install_requires=['logging', 'typing'],
)